@@ -1,4 +1,21 @@
 # -*- coding: utf-8 -*-
+# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
+#
+# This file is part of logilab-common.
+#
+# logilab-common is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 2.1 of the License, or (at your option) any
+# later version.
+#
+# logilab-common is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
 """Run tests.
 
 This will find all modules whose name match a given prefix in the test
@@ -7,21 +24,18 @@ additional facilities.
 
 Command line options:
 
- -v: verbose -- run tests in verbose mode with output to stdout
- -q: quiet   -- don't print anything except if a test fails
- -t: testdir -- directory where the tests will be found
- -x: exclude -- add a test to exclude
- -p: profile -- profiled execution
- -c: capture -- capture standard out/err during tests
- -d: dbc     -- enable design-by-contract
- -m: match   -- only run test matching the tag pattern which follow
+ -v  verbose -- run tests in verbose mode with output to stdout
+ -q  quiet   -- don't print anything except if a test fails
+ -t  testdir -- directory where the tests will be found
+ -x  exclude -- add a test to exclude
+ -p  profile -- profiled execution
+ -c  capture -- capture standard out/err during tests
+ -d  dbc     -- enable design-by-contract
+ -m  match   -- only run test matching the tag pattern which follow
 
 If no non-option arguments are present, prefixes used are 'test',
 'regrtest', 'smoketest' and 'unittest'.
 
-:copyright: 2003-2009 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
-:license: General Public License version 2 - http://www.gnu.org/licenses
 """
 __docformat__ = "restructuredtext en"
 # modified copy of some functions from test/regrtest.py from PyXml
@@ -46,6 +60,7 @@ import warnings
 from compiler.consts import CO_GENERATOR
 from ConfigParser import ConfigParser
 from itertools import dropwhile
+from functools import wraps
 
 try:
     from test import test_support
@@ -81,10 +96,11 @@ __unittest = 1
 def with_tempdir(callable):
     """A decorator ensuring no temporary file left when the function return
     Work only for temporary file create with the tempfile module"""
+    @wraps(callable)
     def proxy(*args, **kargs):
 
         old_tmpdir = tempfile.gettempdir()
-        new_tmpdir = tempfile.mkdtemp("-logilab-common-testlib","temp-dir-")
+        new_tmpdir = tempfile.mkdtemp(prefix="temp-lgc-")
         tempfile.tempdir = new_tmpdir
         try:
             return callable(*args, **kargs)
@@ -95,6 +111,26 @@ def with_tempdir(callable):
                 tempfile.tempdir = old_tmpdir
     return proxy
 
+def in_tempdir(callable):
+    """A decorator moving the enclosed function inside the tempfile.tempfdir
+    """
+    @wraps(callable)
+    def proxy(*args, **kargs):
+
+        old_cwd = os.getcwd()
+        os.chdir(tempfile.tempdir)
+        try:
+            return callable(*args, **kargs)
+        finally:
+            os.chdir(old_cwd)
+    return proxy
+
+def within_tempdir(callable):
+    """A decorator run the enclosed function inside a tmpdir removed after execution
+    """
+    proxy = with_tempdir(in_tempdir(callable))
+    proxy.__name__ = callable.__name__
+    return proxy
 
 def run_tests(tests, quiet, verbose, runner=None, capture=0):
     """Execute a list of tests.
@@ -717,7 +753,7 @@ Examples:
                     self.printonly = re.compile(value)
                 if opt in ('-s', '--skip'):
                     self.skipped_patterns = [pat.strip() for pat in
-                        value.split(', ')]
+                                             value.split(', ')]
                 if opt == '--color':
                     self.colorize = True
                 if opt in ('-m', '--match'):
@@ -808,6 +844,9 @@ Examples:
 succeeded tests into", osp.join(os.getcwd(),FILE_RESTART)
 
         result = self.testRunner.run(self.test)
+        # help garbage collection: we want TestSuite, which hold refs to every
+        # executed TestCase, to be gc'ed
+        del self.test
         if hasattr(self.module, 'teardown_module'):
             try:
                 self.module.teardown_module(self.options, result)
@@ -990,9 +1029,9 @@ class TestCase(unittest.TestCase):
     # instantiated for each test run)
     datadir = classproperty(cached(datadir))
 
-    def datapath(cls, fname):
+    def datapath(cls, *fname):
         """joins the object's datadir and `fname`"""
-        return osp.join(cls.datadir, fname)
+        return osp.join(cls.datadir, *fname)
     datapath = classmethod(datapath)
 
     def set_description(self, descr):
@@ -1328,21 +1367,82 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
         self.assertListEquals(lines1, lines2, msg)
     assertLineEqual = assertLinesEquals
 
-    def assertXMLWellFormed(self, stream, msg=None):
-        """asserts the XML stream is well-formed (no DTD conformance check)"""
-        from xml.sax import make_parser, SAXParseException
-        parser = make_parser()
+    def assertXMLWellFormed(self, stream, msg=None, context=2):
+        """asserts the XML stream is well-formed (no DTD conformance check)
+        :context: number of context lines in standard msg. all data if negativ
+                  only available with element tree
+        """
         try:
-            parser.parse(stream)
-        except SAXParseException:
+            from xml.etree.ElementTree import parse
+            self._assertETXMLWellFormed(stream, parse, msg)
+        except ImportError:
+            from xml.sax import make_parser, SAXParseException
+            parser = make_parser()
+            try:
+                parser.parse(stream)
+            except SAXParseException, ex:
+                if msg is None:
+                    stream.seek(0)
+                    for _ in xrange(ex.getLineNumber()):
+                        line = stream.readline()
+                    pointer = ('' * (ex.getLineNumber() - 1)) + '^'
+                    msg = 'XML stream not well formed: %s\n%s%s' % (ex, line, pointer)
+                self.fail(msg)
+
+    def assertXMLStringWellFormed(self, xml_string, msg=None, context=2):
+        """asserts the XML string is well-formed (no DTD conformance check)
+        :context: number of context lines in standard msg. all data if negativ
+                  only available with element tree
+        """
+        try:
+            from xml.etree.ElementTree import fromstring
+            self._assertETXMLWellFormed(xml_string, fromstring, msg)
+        except ImportError:
+            raise
+            stream = StringIO(xml_string)
+            self.assertXMLWellFormed(stream, msg)
+
+    def _assertETXMLWellFormed(self, data, parse, msg=None, context=2):
+        """internal function used by /assertXML(String)?WellFormed/ functions
+        :data: xml_data
+        :parse: appropriate parser function for this data
+        :msg: error message
+        :context: number of context lines in standard msg. all data if negativ
+                  only available with element tree
+        """
+        from xml.parsers.expat import ExpatError
+        try:
+            parse(data)
+        except ExpatError, ex:
             if msg is None:
-                msg = 'XML stream not well formed'
+                if hasattr(data, 'readlines'): #file like object
+                    stream.seek(0)
+                    lines = stream.readlines()
+                else:
+                    lines =data.splitlines(True)
+                nb_lines = len(lines)
+                context_lines = []
+
+                if  context < 0:
+                    start = 1
+                    end   = nb_lines
+                else:
+                    start = max(ex.lineno-context, 1)
+                    end   = min(ex.lineno+context, nb_lines)
+                line_number_length = len('%i' % end)
+                line_pattern = " %%%ii: %%s" % line_number_length
+
+                for line_no in xrange(start, ex.lineno):
+                    context_lines.append(line_pattern % (line_no, lines[line_no-1]))
+                context_lines.append(line_pattern % (ex.lineno, lines[ex.lineno-1]))
+                context_lines.append('%s^\n' % (' ' * (1 + line_number_length + 2 +ex.offset)))
+                for line_no in xrange(ex.lineno+1, end+1):
+                    context_lines.append(line_pattern % (line_no, lines[line_no-1]))
+
+                rich_context = ''.join(context_lines)
+                msg = 'XML stream not well formed: %s\n%s' % (ex, rich_context)
             self.fail(msg)
 
-    def assertXMLStringWellFormed(self, xml_string, msg=None):
-        """asserts the XML string is well-formed (no DTD conformance check)"""
-        stream = StringIO(xml_string)
-        self.assertXMLWellFormed(stream, msg)
 
     def assertXMLEqualsTuple(self, element, tup):
         """compare an ElementTree Element to a tuple formatted as follow:
@@ -1391,7 +1491,7 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
                 self.fail('\n'.join(['%s\n'%msg_prefix]+read + list(result)))
 
     def assertTextEquals(self, text1, text2, junk=None,
-            msg_prefix='Text differ'):
+            msg_prefix='Text differ', striplines=False):
         """compare two multiline strings (using difflib and splitlines())"""
         msg = []
         if not isinstance(text1, basestring):
@@ -1400,8 +1500,12 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
             msg.append('text2 is not a string (%s)'%(type(text2)))
         if msg:
             self.fail('\n'.join(msg))
-        self._difftext(text1.strip().splitlines(True), text2.strip().splitlines(True),
-                       junk,  msg_prefix)
+        lines1 = text1.strip().splitlines(True)
+        lines2 = text2.strip().splitlines(True)
+        if striplines:
+            lines1 = [line.strip() for line in lines1]
+            lines2 = [line.strip() for line in lines2]
+        self._difftext(lines1, lines2, junk,  msg_prefix)
     assertTextEqual = assertTextEquals
 
     def assertStreamEquals(self, stream1, stream2, junk=None,
